import chimera, numpy as np, Tkinter as tk
from chimera.baseDialog import ModelessDialog
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
from StructBio.Utilities.miscFunctions import tickList

#-------------------------------------------------------------------------
class CurvePlotBundle(object):
    def __init__(self, xyPts_L, color = "black", legendEntry = ""):
        unzippedData = zip(*xyPts_L)
        self.x_L = unzippedData[0]
        self.y_L = unzippedData[1]
        self.color = color
        self.legendEntry = legendEntry
        self.xMin = min(self.x_L)
        self.xMax = max(self.x_L)
        self.yMin = min(self.y_L)
        self.yMax = max(self.y_L)
        self.showInPlot = False # Used when a plot bundle is to be ignored.
        
#-------------------------------------------------------------------------
class GUIwithPlot(ModelessDialog):
    buttons = ("Close")
    help = False    # Leave this out to get a Help button.

    def fillInUI(self, parent, title = ""):
        self.plotBundles_L = []
        self.bundleIndex = 0
        self.chkBtnVars_D = {} # Vars for check buttons in control frame.
        # The widgets in a "control" frame on the left are used to determine plot content.
        # The "plot" frame on the right holds the plot itself.
        # A "legend" frame appears just below the control frame.
        self.controlFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        self.controlFrame.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.N + tk.W + tk.E)

        self.plotFrame = tk.Frame(parent)
        self.plotFrame.grid(row = 0, column = 1, rowspan = 2, padx = 4, pady = 4, sticky = tk.N)

        self.legendFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        self.legendFrame.grid(row = 1, column = 0, padx = 4, pady = 4, sticky = tk.S)
        
        # Text to be displayed at top of control frame:
        topText = tk.Label(self.controlFrame, text =  " Generate plot:",
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 1, pady = 4, sticky = tk.N)



    def setupPlot(self, plotTitle = "", xAxisLabel = "", yAxisLabel = "", gridFlag = False,
                  margin = 0.1, aspect = 1.0, numTicks = 10):
        self.plotTitle = plotTitle
        self.xAxisLabel = xAxisLabel
        self.yAxisLabel = yAxisLabel
        self.gridFlag = gridFlag
        self.margin = margin        # Margin around curve being plotted.
        self.aspect = aspect
        self.numTicks = numTicks    # Number of ticks for longest side of plot.
        self.fig = Figure(figsize = (6,6), dpi = 100)
        self.figSp = self.fig.add_subplot(111, aspect = self.aspect)       
        
    def addPlot(self, plotBundle):
        self.plotBundles_L.append(plotBundle)

        # Put into the control frame a checkbox corresponding to this bundle.
        self.chkBtnVars_D[self.bundleIndex] = tk.BooleanVar()
        # Note use of lambda to avoid creation of multiple callback functions:
        tk.Checkbutton(self.controlFrame, text = "Curve " + str(self.bundleIndex),
                       variable = self.chkBtnVars_D[self.bundleIndex],
                       command = lambda arg = self.bundleIndex: self.chkBtnFunc(arg), indicatoron = 1).   \
                       grid(row = 1 + self.bundleIndex, column = 0, padx = 2, sticky = tk.W)       
        self.bundleIndex += 1        
        
    #-------------------------------------------------------------------------
    # Callback function needed for Buttons:
    def chkBtnFunc(self, buttonSpec):
        self.figSp.clear() # Clear previous plot before generating a new plot.
        
        # Get the button variable and use it to specify whether the corresponding
        # plot bundle should produce a plot:
        self.plotBundles_L[buttonSpec].showInPlot = self.chkBtnVars_D[buttonSpec].get()
        
        #-------------------------------------------------------------------------
        # Set up a canvas for the plot legend.
        self.legendCanvas = tk.Canvas(self.legendFrame, width = 150, height = 300,
                                      background = "white")
        self.legendCanvas.grid(row = 0, column = 0)
        self.legendCanvas.create_text((2,2), text = "Legend:", anchor = tk.NW)

        # Select the plot bundles that are to be shown:        
        selectedPlotBundles_L = [pB for pB in self.plotBundles_L if pB.showInPlot]
        
        # Determin the plot limits and disposition of ticks:
        if len(selectedPlotBundles_L) > 0:
            xMin = min(pB.xMin for pB in selectedPlotBundles_L) - self.margin
            xMax = max(pB.xMax for pB in selectedPlotBundles_L) + self.margin
            yMin = min(pB.yMin for pB in selectedPlotBundles_L) - self.margin
            yMax = max(pB.yMax for pB in selectedPlotBundles_L) + self.margin
            longestDimension = max(xMax - xMin, yMax - yMin)
            numXticks = int(self.numTicks*(xMax - xMin)/longestDimension)
            numYticks = int(self.numTicks*(yMax - yMin)/longestDimension)
            xTickList = tickList(xMin, xMax, numXticks)
            yTickList = tickList(yMin, yMax, numYticks)
            self.figSp.set_xlim([xTickList[0], xTickList[-1]])
            self.figSp.set_ylim([yTickList[0], yTickList[-1]])
            self.figSp.set_xticks(np.array(xTickList))
            self.figSp.set_yticks(np.array(yTickList))

        # Specify fontsize of tick lables: and whether the grid should appear:                
        self.figSp.grid(self.gridFlag)
        plt.setp(self.figSp.get_yticklabels(), fontsize = 10)
        plt.setp(self.figSp.get_xticklabels(), fontsize = 10)
        
        #-------------------------------------------------------------------------
        # Set up a plot canvas and put self.fig into it.
        plotCanvas = FigureCanvasTkAgg(self.fig, master = self.plotFrame)
        plotCanvas.get_tk_widget().grid(row = 0, column = 0)
        
        self.figSp.set_title(self.plotTitle, fontsize = 14)
        self.figSp.set_xlabel(self.xAxisLabel, fontsize = 12, style = 'italic')
        self.figSp.set_ylabel(self.yAxisLabel, fontsize = 12, style = 'italic') 
        
        if len(selectedPlotBundles_L) == 0: # Remove ticks if the plot is empty.
            self.figSp.set_xticks([])
            self.figSp.set_yticks([]) 

        # Put in plots for selected plot bundles:
        legendPos = 0
        for pB in selectedPlotBundles_L:
            self.figSp.plot(pB.x_L, pB.y_L, color = pB.color)
            
            legendPos += 1
            self.legendCanvas.create_text((2, legendPos*20), text = pB.legendEntry,
                                          fill = pB.color, anchor = tk.NW)
                            
#===============================================================================
# Mainline for testing:        
wb = GUIwithPlot(title = "Parametric Curve Plots")
wb.setupPlot("Parametric Curves", "x", "y", gridFlag = True, numTicks = 12)

# Generate parametric representation of curves:
circle_L = []
for t in np.arange(0.0, 101.0, 1.0):
    theta = 2.0*np.pi*t/100.
    circle_L.append((6.0*np.pi + 4.0*np.sin(theta), 15. + 4.0*np.cos(theta)))
wb.addPlot(CurvePlotBundle(circle_L, color = "blue", legendEntry = "Circle"))


trochoid_L = []
for t in np.arange(0.0, 301.0, 1.0):
    theta = 2.0*np.pi*t/100.
    trochoid_L.append((2.0*theta - 3.0*np.sin(theta), 2.0 - 3.0*np.cos(theta)))
wb.addPlot(CurvePlotBundle(trochoid_L, color = "green", legendEntry = "Trochoid"))


cardioid_L = []
for t in np.arange(0.0, 201.0, 1.0):
    theta = 2.0*np.pi*t/100.
    cardioid_L.append((6.0*np.pi + 3.0*(2.0*np.sin(theta) - np.sin(2.0*theta)),
                       6.0 + 3.0*(2.0*np.cos(theta) - np.cos(2.0*theta))))
wb.addPlot(CurvePlotBundle(cardioid_L, color = "red", legendEntry = "Cardioid"))













